package at.ac.uibk.gitsearch.service;

import java.util.List;

import javax.persistence.criteria.JoinType;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import io.github.jhipster.service.QueryService;

import at.ac.uibk.gitsearch.domain.Likes;
import at.ac.uibk.gitsearch.domain.*; // for static metamodels
import at.ac.uibk.gitsearch.repository.LikesRepository;
import at.ac.uibk.gitsearch.repository.search.LikesSearchRepository;
import at.ac.uibk.gitsearch.service.dto.LikesCriteria;

/**
 * Service for executing complex queries for {@link Likes} entities in the database.
 * The main input is a {@link LikesCriteria} which gets converted to {@link Specification},
 * in a way that all the filters must apply.
 * It returns a {@link List} of {@link Likes} or a {@link Page} of {@link Likes} which fulfills the criteria.
 */
@Service
@Transactional(readOnly = true)
public class LikesQueryService extends QueryService<Likes> {

    private final Logger log = LoggerFactory.getLogger(LikesQueryService.class);

    private final LikesRepository likesRepository;

    private final LikesSearchRepository likesSearchRepository;

    public LikesQueryService(LikesRepository likesRepository, LikesSearchRepository likesSearchRepository) {
        this.likesRepository = likesRepository;
        this.likesSearchRepository = likesSearchRepository;
    }

    /**
     * Return a {@link List} of {@link Likes} which matches the criteria from the database.
     * @param criteria The object which holds all the filters, which the entities should match.
     * @return the matching entities.
     */
    @Transactional(readOnly = true)
    public List<Likes> findByCriteria(LikesCriteria criteria) {
        log.debug("find by criteria : {}", criteria);
        final Specification<Likes> specification = createSpecification(criteria);
        return likesRepository.findAll(specification);
    }

    /**
     * Return a {@link Page} of {@link Likes} which matches the criteria from the database.
     * @param criteria The object which holds all the filters, which the entities should match.
     * @param page The page, which should be returned.
     * @return the matching entities.
     */
    @Transactional(readOnly = true)
    public Page<Likes> findByCriteria(LikesCriteria criteria, Pageable page) {
        log.debug("find by criteria : {}, page: {}", criteria, page);
        final Specification<Likes> specification = createSpecification(criteria);
        return likesRepository.findAll(specification, page);
    }

    /**
     * Return the number of matching entities in the database.
     * @param criteria The object which holds all the filters, which the entities should match.
     * @return the number of matching entities.
     */
    @Transactional(readOnly = true)
    public long countByCriteria(LikesCriteria criteria) {
        log.debug("count by criteria : {}", criteria);
        final Specification<Likes> specification = createSpecification(criteria);
        return likesRepository.count(specification);
    }

    /**
     * Function to convert {@link LikesCriteria} to a {@link Specification}
     * @param criteria The object which holds all the filters, which the entities should match.
     * @return the matching {@link Specification} of the entity.
     */
    protected Specification<Likes> createSpecification(LikesCriteria criteria) {
        Specification<Likes> specification = Specification.where(null);
        if (criteria != null) {
            if (criteria.getId() != null) {
                specification = specification.and(buildRangeSpecification(criteria.getId(), Likes_.id));
            }
            if (criteria.getDate() != null) {
                specification = specification.and(buildRangeSpecification(criteria.getDate(), Likes_.date));
            }
            if (criteria.getUserID() != null) {
                specification = specification.and(buildRangeSpecification(criteria.getUserID(), Likes_.userID));
            }
            if (criteria.getProjectID() != null) {
                specification = specification.and(buildRangeSpecification(criteria.getProjectID(), Likes_.projectID));
            }
        }
        return specification;
    }
}
