package at.ac.uibk.gitsearch.service;

import java.util.List;

import javax.persistence.criteria.JoinType;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import io.github.jhipster.service.QueryService;

import at.ac.uibk.gitsearch.domain.UserWatchList;
import at.ac.uibk.gitsearch.domain.*; // for static metamodels
import at.ac.uibk.gitsearch.repository.UserWatchListRepository;
import at.ac.uibk.gitsearch.service.dto.UserWatchListCriteria;
import at.ac.uibk.gitsearch.service.dto.UserWatchListDTO;
import at.ac.uibk.gitsearch.service.mapper.UserWatchListMapper;

/**
 * Service for executing complex queries for {@link UserWatchList} entities in the database.
 * The main input is a {@link UserWatchListCriteria} which gets converted to {@link Specification},
 * in a way that all the filters must apply.
 * It returns a {@link List} of {@link UserWatchListDTO} or a {@link Page} of {@link UserWatchListDTO} which fulfills the criteria.
 */
@Service
@Transactional(readOnly = true)
public class UserWatchListQueryService extends QueryService<UserWatchList> {

    private final Logger log = LoggerFactory.getLogger(UserWatchListQueryService.class);

    private final UserWatchListRepository userWatchListRepository;

    private final UserWatchListMapper userWatchListMapper;

    public UserWatchListQueryService(UserWatchListRepository userWatchListRepository, UserWatchListMapper userWatchListMapper) {
        this.userWatchListRepository = userWatchListRepository;
        this.userWatchListMapper = userWatchListMapper;
    }

    /**
     * Return a {@link List} of {@link UserWatchListDTO} which matches the criteria from the database.
     * @param criteria The object which holds all the filters, which the entities should match.
     * @return the matching entities.
     */
    @Transactional(readOnly = true)
    public List<UserWatchListDTO> findByCriteria(UserWatchListCriteria criteria) {
        log.debug("find by criteria : {}", criteria);
        final Specification<UserWatchList> specification = createSpecification(criteria);
        return userWatchListMapper.toDto(userWatchListRepository.findAll(specification));
    }

    /**
     * Return a {@link Page} of {@link UserWatchListDTO} which matches the criteria from the database.
     * @param criteria The object which holds all the filters, which the entities should match.
     * @param page The page, which should be returned.
     * @return the matching entities.
     */
    @Transactional(readOnly = true)
    public Page<UserWatchListDTO> findByCriteria(UserWatchListCriteria criteria, Pageable page) {
        log.debug("find by criteria : {}, page: {}", criteria, page);
        final Specification<UserWatchList> specification = createSpecification(criteria);
        return userWatchListRepository.findAll(specification, page)
            .map(userWatchListMapper::toDto);
    }

    /**
     * Return the number of matching entities in the database.
     * @param criteria The object which holds all the filters, which the entities should match.
     * @return the number of matching entities.
     */
    @Transactional(readOnly = true)
    public long countByCriteria(UserWatchListCriteria criteria) {
        log.debug("count by criteria : {}", criteria);
        final Specification<UserWatchList> specification = createSpecification(criteria);
        return userWatchListRepository.count(specification);
    }

    /**
     * Function to convert {@link UserWatchListCriteria} to a {@link Specification}
     * @param criteria The object which holds all the filters, which the entities should match.
     * @return the matching {@link Specification} of the entity.
     */
    protected Specification<UserWatchList> createSpecification(UserWatchListCriteria criteria) {
        Specification<UserWatchList> specification = Specification.where(null);
        if (criteria != null) {
            if (criteria.getId() != null) {
                specification = specification.and(buildRangeSpecification(criteria.getId(), UserWatchList_.id));
            }
            if (criteria.getName() != null) {
                specification = specification.and(buildStringSpecification(criteria.getName(), UserWatchList_.name));
            }
            if (criteria.getUserIdId() != null) {
                specification = specification.and(buildSpecification(criteria.getUserIdId(),
                    root -> root.join(UserWatchList_.userId, JoinType.LEFT).get(User_.id)));
            }
        }
        return specification;
    }
}
